package com.hzya.frame.plugin.oa.poorder.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.alibaba.fastjson.serializer.SerializeConfig;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.alibaba.fastjson.serializer.SimplePropertyPreFilter;
import com.hzya.frame.plugin.oa.poorder.service.IPoOrderService;
import com.hzya.frame.seeyon.util.OARestUtil;
import com.hzya.frame.sysnew.application.entity.SysExtensionApiEntity;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * @Description 接收NCC采购订单的请求
 * @Author xiangerlin
 * @Date 2025/6/12 17:04
 **/
@Service(value="nCCPoOrderServiceImpl")
public class PoOrderServiceImpl implements IPoOrderService {
    Logger logger = LoggerFactory.getLogger(PoOrderServiceImpl.class);
    @Autowired
    private OARestUtil oaRestUtil;

    @Value("${zt.url}")
    private String interfaceUrl;
    /**
     * 接收NCC的请求，并转换成OA的参数
     *
     * @param reqJson
     * @return
     */
    @Override
    public JsonResultEntity buildOAParam(JSONObject reqJson) {
        JSONObject nccObj = JSONObject.parseObject(reqJson.getString("jsonStr"));
        logger.info("中台接收到NCC采购订单的请求参数:{}",reqJson.getString("jsonStr"));
        Assert.notNull(nccObj,"参数不能为空");
        JSONObject head = nccObj.getJSONObject("head");

        JSONArray detailList = nccObj.getJSONArray("body");
        Assert.notNull(head,"参数不能为空");
        Assert.notNull(detailList,"参数不能为空");
        head.put("user_code","seeyon");
        //组装参数
        JSONObject formmain_0256 = new JSONObject();//主表数据
        List<JSONObject> formson_0257_list = new LinkedList<>();//子表数据
        formmain_0256.put("采购组织",head.getString("org_name"));
        formmain_0256.put("订单编号",head.getString("vbillcode"));
        formmain_0256.put("订单日期",head.getString("dbilldate"));
        formmain_0256.put("供应商",head.getString("sup_name"));
        formmain_0256.put("采购员",head.getString("cemployeename"));
        formmain_0256.put("开票供应商",head.getString("invc_sup_name"));
        formmain_0256.put("付款协议",head.getString("payterm_name"));
        formmain_0256.put("单据状态",head.getString("forderstatus"));
        formmain_0256.put("备注",head.getString("vmemo"));
        formmain_0256.put("补货",head.getString("bisreplenish"));
        formmain_0256.put("退货",head.getString("breturn"));
        formmain_0256.put("总数量",head.getString("ntotalastnum"));
        formmain_0256.put("总价税合计",head.getString("ntotalorigmny"));
        formmain_0256.put("币种",head.getString("currtype_name"));
        formmain_0256.put("退货退库基于原订单补货",head.getString("brefwhenreturn"));
        formmain_0256.put("最终关闭",head.getString("bfinalclose"));
        formmain_0256.put("采购部门",head.getString("dept_name"));
        formmain_0256.put("订单类型",head.getString("BILLTYPENAME"));
        //子表
        for (int i=0; i < detailList.size(); i++){
            JSONObject detail = (JSONObject) detailList.get(i);
            JSONObject formson_0257 = new JSONObject();
            formson_0257.put("收货库存组织",detail.getString("arrvstoorg_name").toLowerCase());
            formson_0257.put("行号",detail.getString("crowno"));
            formson_0257.put("物料编码",detail.getString("material_code"));
            formson_0257.put("物料名称",detail.getString("material_name"));
            formson_0257.put("单位",detail.getString("cunitname"));
            formson_0257.put("数量",detail.getString("nnum"));
            formson_0257.put("换算率",detail.getString("vchangerate"));
            formson_0257.put("无税单价",detail.getString("nqtorigprice"));
            formson_0257.put("含税单价",detail.getString("nqtorigtaxprice"));
            formson_0257.put("集团本币无税金额",detail.getString("norigmny"));
            formson_0257.put("集团本币价税合计",detail.getString("norigtaxmny"));
            formson_0257.put("计划收货日期",detail.getString("dplanarrvdate"));
            formson_0257.put("收货仓库",detail.getString("recvstordoc_name"));
            formson_0257.put("税率",detail.getString("ntaxrate"));
            formson_0257_list.add(formson_0257);
        }
        String param = oaRestUtil.processParamValueOf(formmain_0256, formson_0257_list, "formmain_0256_dev", "formmain_0256", "formson_0257");
        String token = oaRestUtil.getToken(head.getString("user_code"), "8000640001");
        logger.info("NCC发起OA采购订单请求参数{}",param);
        //调用OA接口
        String result = HttpRequest.post(interfaceUrl)
                .header("token",token)// token
                .header("appId","800064")// 建辉OA应用
                .header("apiCode","8000640000")//流程表单接口
                .header("publicKey","ZJYAAr9TeFduBYu7uJeie2KYdQsnBaEYZHmOjPWMMVZmzelCe7eDIk+3zDUT+v578prj")//NCC应用key
                .header("secretKey","f4SSnYcDQmdkwwuGJd2+934q6lM1NnUm5dmOw/4Wvy2mo6PgkKsI/drXYyyLWQCaj3JzOP8MtA1LSGvL+2BWG8c/o7DKi92S4mr3zcGearA=")//NCC应用密钥
                .body(param).execute().body();
        // JsonResultEntity resultEntity = (JsonResultEntity) oerDjmlService.handleOerDjml(paramJSON);
        // logger.info("保存GRPU8响应参数:{}",JSONObject.toJSON(resultEntity));
        logger.info("NCC发起OA采购订单响应参数:{}",result);
        JSONObject resultJson = JSONObject.parseObject(result);
        if (null == resultJson){
            return BaseResult.getFailureMessageEntity("发起OA审批失败");
        }
        if (!resultJson.getBoolean("flag")){
            return BaseResult.getFailureMessageEntity("发起OA审批失败"+resultJson.getString("msg"));
        }
        return BaseResult.getSuccessMessageEntity(resultJson.getJSONObject("attribute"));
    }

    private Object convertLowerCase(JSONObject jsonObject,String key){
        if (null != jsonObject && StrUtil.isNotEmpty(key)){
           return null == jsonObject.get(key.toUpperCase()) ? jsonObject.get(key.toLowerCase()) : jsonObject.get(key.toUpperCase());
        }
        return null;
    }
}
